; ########################################
; OPENSAVEDIALOG
; V 19 (10.03.2013)
; (C) BY NILS HINTZE
; http://www.elektronikpage.net/purebasic/
; ########################################



#OFN_READONLY           = $00000001
#OFN_OVERWRITEPROMPT    = $00000002
#OFN_HIDEREADONLY       = $00000004
#OFN_ALLOWMULTISELECT   = $00000200
#OFN_FILEMUSTEXIST      = $00001000
#OFN_EXPLORER           = $00080000
#OFN_NODEREFERENCELINKS = $00100000
#OFN_DONTADDTORECENT    = $02000000

#OpenFileDialog_DefaultTitle           = $01
#OpenFileDialog_AddToRecent            = $02
#OpenFileDialog_ReadOnlyCheckBox       = $04
#OpenFileDialog_ReadOnlyChecked        = $08
#OpenFileDialog_FileDoesntExistWarning = $10
#OpenFileDialog_NoDereferenceLnk       = $20
#OpenFileDialog_MultiSelection         = $40
#OpenFileDialog_FileBufferSize = 10000

#SaveFileDialog_DefaultTitle     = $01
#SaveFileDialog_AddToRecent      = $02
#SaveFileDialog_OverwriteWarning = $04
#SaveFileDialog_NoDereferenceLnk = $08
#SaveFileDialog_FileBufferSize = 1000

Global OpenFileDialog_SelectedPatternIndex, OpenFileDialog_ReadOnlyChecked, NewList OpenFileDialog_FileList.s(), SaveFileDialog_SelectedPatternIndex

Macro OpenFileDialogSelectedPatternIndex()
  (OpenFileDialog_SelectedPatternIndex)
EndMacro

Macro OpenFileDialogReadOnlyChecked()
  (OpenFileDialog_ReadOnlyChecked)
EndMacro

Procedure.s OpenFileDialogNextSelectedFileName()
  If NextElement(OpenFileDialog_FileList())
    ProcedureReturn OpenFileDialog_FileList()
  EndIf
EndProcedure

Macro SaveFileDialogSelectedPatternIndex()
  (SaveFileDialog_SelectedPatternIndex)
EndMacro



; ########################################
; OpenFileDialog()
; Zeigt ein ffnen-Dialogfenster an.
; 
; Title$ ist der Fenstertitel. DefaultFile$ ist der vorgegebene Dateiname.
; Pattern$ enthlt die Filter. Syntax wie bei OpenFileRequester(). Beispiel:
; Pattern$ = "Text-Dateien|*.txt|HTML-Dateien|*.htm;*.html|Alle Dateien|*.*"
; DefaultPattern ist die Nummer des vorgegebenen Filter, beginnend bei 0.
; ParentWindowID ist die WindowID des bergeordneten Fensters und kann #Null
; sein, wenn kein Fenster vorhanden ist. DefaultExtension$ ist die vorgegebene
; Erweiterung (siehe unten). Flags kann eine Kombination der nachfolgend
; beschriebenen Flags sein. Rckgabewert ist der Dateiname bei Erfolg,
; andernfalls ein leerer String.
; 
; Mit OpenFileDialogSelectedPatternIndex() kann die Nummer des Patterns
; ermittelt werden, welches der Benutzer ausgewhlt hat.
; 
; Wenn der Benutzer einen Dateinamen ohne Erweiterung angibt, wird automatisch
; die Erweiterung des ausgewhlten Patterns angefgt. Wenn im gewhlten Pattern
; mehrere Erweiterungen angegeben sind (durch Semikolon getrennt), wird die
; erste Erweiterung angefgt. Wenn im Pattern keine eindeutige Erweiterung (z.B.
; *.*) angegeben ist, wird die Erweiterung in DefaultExtension$ angefgt, wobei
; nur die ersten drei Zeichen von DefaultExtension$ verwendet werden. Wenn
; DefaultExtension$ ein leerer String ist, wird grundstzlich nichts angefgt.
; 
; Mit dem Flag #OpenFileDialog_DefaultTitle wird der in Title$ angegebene
; Fenstertitel ignoriert und ein vom Betriebssystem vorgegebener Titel
; verwendet.
; 
; Mit dem Flag #OpenFileDialog_AddToRecent wird die ausgewhlte Datei zur Liste
; der zuletzt benutzten Dateien hinzugefgt.
; 
; Mit dem Flag #OpenFileDialog_ReadOnlyCheckBox wird eine CheckBox mit dem Text
; "Schreibgeschtzt ffnen" angezeigt. Mit #OpenFileDialog_ReadOnlyChecked kann
; der Anfangszustand der CheckBox festgelegt werden. Die Funktion
; OpenFileDialogReadOnlyChecked() gibt #True zurck, wenn der Benutzer die
; CheckBox aktiviert hat, oder #False, wenn dies nicht der Fall war.
; 
; Mit dem Flag #OpenFileDialog_FileDoesntExistWarning wird eine Warnung
; angezeigt und das Dialogfenster nicht geschlossen, wenn der Benutzer versucht,
; eine nicht existierende Datei zu ffnen. Wenn dieses Flag nicht angegeben
; wird, gibt die Funktion auch Dateinamen zurck, welche nicht existieren.
; 
; Wenn das Flag #OpenFileDialog_NoDereferenceLnk angegeben ist und der Benutzer
; eine Verknpfung auswhlt, wird der Dateiname der Verknpfungsdatei (.lnk)
; zurckgegeben. Wird das Flag nicht angegeben, so wird der Name der Zieldatei
; der Verknpfung zurckgegeben.
; 
; Mit dem Flag #OpenFileDialog_MultiSelection kann der Benutzer mehrere Dateien
; auswhlen. OpenFileDialog() gibt dann den Namen der ersten ausgewhlten Datei
; zurck. Mittels der Funktion OpenFileDialogNextSelectedFileName() knnen die
; weiteren Dateinamen ermittelt werden. Diese Funktion gibt bei jedem Aufruf
; einen weiteren Dateinamen zurck. Wenn keine weiteren Dateinamen mehr
; vorhanden sind, wird ein leerer String zurckgegeben. Wenn dieses Flag
; angegeben wird und der Benutzer eine Verknpfung auswhlt, wird der Dateiname
; der Verknpfungsdatei (*.lnk) zurckgegeben, wie bei Angabe des Flags
; #OpenFileDialog_NoDereferenceLnk.
; ########################################
Procedure.s OpenFileDialog(Title$, DefaultFile$, Pattern$, DefaultPattern, ParentWindowID = #Null, DefaultExtension$ = "", Flags = 0)
  
  Protected *FilterBuf, *FileBuf, *c.Character, OFN.OPENFILENAME, Path$
  
  ; Alte Dateinamen lschen
  ClearList(OpenFileDialog_FileList())
  
  ; Speicher fr Filter
  *FilterBuf = AllocateMemory((Len(Pattern$)+2)*SizeOf(Character)) ; Mit Len(Pattern$) + 2 wird sichergestellt, dass lpstrFilter mit zwei Null-Zeichen terminiert wird.
  If Not *FilterBuf
    Goto OpenFileDialog_Error
  EndIf  
  PokeS(*FilterBuf, Pattern$)
  *c = *FilterBuf
  While *c\c
    If *c\c = '|'
      *c\c = 0
    EndIf
    *c + SizeOf(Character)
  Wend

  ; Speicher fr Dateiname
  *FileBuf = AllocateMemory(#OpenFileDialog_FileBufferSize*SizeOf(Character))
  If Not *FileBuf
    Goto OpenFileDialog_Error
  EndIf
  PokeS(*FileBuf, DefaultFile$, #OpenFileDialog_FileBufferSize-2) ; Mit #OpenFileDialog_FileBufferSize - 2 wird sichergestellt, dass lpstrFile mit zwei Null-Zeichen terminiert wird.
  
  ; Struktur initialisieren
  With OFN
    \lStructSize = SizeOf(OPENFILENAME)
    \hwndOwner = ParentWindowID
    \lpstrFilter = *FilterBuf
    \lpstrCustomFilter = #Null
    \nFilterIndex = DefaultPattern + 1
    \lpstrFile = *FileBuf
    \nMaxFile = #OpenFileDialog_FileBufferSize - 1 ; Mit #OpenFileDialog_FileBufferSize - 1 wird sichergestellt, dass lpstrFile mit zwei Null-Zeichen terminiert wird.
    \lpstrFileTitle = #Null
    \lpstrInitialDir = #Null
    If Flags & #OpenFileDialog_DefaultTitle
      \lpstrTitle = #Null
    Else
      \lpstrTitle = @Title$
    EndIf
    If Not Flags & #OpenFileDialog_AddToRecent
      \Flags | #OFN_DONTADDTORECENT
    EndIf
    If Not Flags & #OpenFileDialog_ReadOnlyCheckbox
      \Flags | #OFN_HIDEREADONLY
    ElseIf Flags & #OpenFileDialog_ReadOnlyChecked
      \Flags | #OFN_READONLY
    EndIf
    If Flags & #OpenFileDialog_FileDoesntExistWarning
      \Flags | #OFN_FILEMUSTEXIST
    EndIf
    If Flags & #OpenFileDialog_NoDereferenceLnk
      \Flags | #OFN_NODEREFERENCELINKS
    EndIf
    If Flags & #OpenFileDialog_MultiSelection
      \Flags | #OFN_ALLOWMULTISELECT | #OFN_EXPLORER | #OFN_NODEREFERENCELINKS
    EndIf
    If DefaultExtension$ <> ""
      \lpstrDefExt = @DefaultExtension$
    Else
      \lpstrDefExt = #Null
    EndIf
  EndWith
  
  ; Dialog anzeigen
  If Not GetOpenFileName_(@OFN)
    Goto OpenFileDialog_Error
  EndIf

  ; Pattern ermitteln
  OpenFileDialog_SelectedPatternIndex = OFN\nFilterIndex - 1
  
  ; Dateinamen holen, lpstrFile muss mit zwei Null-Zeichen terminiert werden!
  *c = *FileBuf
  Path$ = PeekS(*c)
  *c + (MemoryStringLength(*c) + 1) * SizeOf(Character)
  While *c\c
    If AddElement(OpenFileDialog_FileList())
      OpenFileDialog_FileList() = Path$ + "\" + PeekS(*c)
    EndIf
    *c + (MemoryStringLength(*c) + 1) * SizeOf(Character)
  Wend
  If ListSize(OpenFileDialog_FileList()) = 0
    If AddElement(OpenFileDialog_FileList())
      OpenFileDialog_FileList() = Path$
    EndIf
  EndIf
  
  ; Readonly-Flag holen
  If OFN\Flags & #OFN_READONLY
    OpenFileDialog_ReadOnlyChecked = #True
  Else
    OpenFileDialog_ReadOnlyChecked = #False
  EndIf

OpenFileDialog_Error:

  ; Speicher freigeben
  If *FilterBuf
    FreeMemory(*FilterBuf)
  EndIf
  If *FileBuf
    FreeMemory(*FileBuf)
  EndIf
  
  ; Ersten Dateinamen zurckgeben
  If FirstElement(OpenFileDialog_FileList())
    ProcedureReturn OpenFileDialog_FileList()
  EndIf

EndProcedure



; ########################################
; SaveFileDialog()
; Zeigt ein Speichern-Dialogfenster an.
;
; Title$ ist der Fenstertitel. DefaultFile$ ist der vorgegebene Dateiname.
; Pattern$ enthlt die Filter. Syntax wie bei OpenFileRequester(). Beispiel:
; Pattern$ = "Text-Dateien|*.txt|HTML-Dateien|*.htm;*.html|Alle Dateien|*.*"
; DefaultPattern ist die Nummer des vorgegebenen Filter, beginnend bei 0.
; ParentWindowID ist die WindowID des bergeordneten Fensters und kann #Null
; sein, wenn kein Fenster vorhanden ist. DefaultExtension$ ist die vorgegebene
; Erweiterung (siehe unten). Flags kann eine Kombination der nachfolgend
; beschriebenen Flags sein. Rckgabewert ist der Dateiname bei Erfolg,
; andernfalls ein leerer String.
;
; Mit SaveFileDialogSelectedPatternIndex() kann die Nummer des Patterns
; ermittelt werden, welches der Benutzer ausgewhlt hat.
;
; Wenn der Benutzer einen Dateinamen ohne Erweiterung angibt, wird automatisch
; die Erweiterung des ausgewhlten Patterns angefgt. Wenn im gewhlten Pattern
; mehrere Erweiterungen angegeben sind (durch Semikolon getrennt), wird die
; erste Erweiterung angefgt. Wenn im Pattern keine eindeutige Erweiterung (z.B.
; *.*) angegeben ist, wird die Erweiterung in DefaultExtension$ angefgt, wobei
; nur die ersten drei Zeichen von DefaultExtension$ verwendet werden. Wenn
; DefaultExtension$ ein leerer String ist, wird grundstzlich nichts angefgt.
;
; Mit dem Flag #SaveFileDialog_DefaultTitle wird der in Title$ angegebene
; Fenstertitel ignoriert und ein vom Betriebssystem vorgegebener Titel
; verwendet.
; 
; Mit dem Flag #SaveFileDialog_AddToRecent wird die ausgewhlte Datei zur Liste
; der zuletzt benutzten Dateien hinzugefgt.
; 
; Mit dem Flag #SaveFileDialog_OverwriteWarning wird vom Benutzer eine
; Besttigung zum berschreiben verlangt, wenn er eine existierende Datei
; auswhlt.
;
; Wenn das Flag #SaveFileDialog_NoDereferenceLnk angegeben ist und der Benutzer
; eine Verknpfung auswhlt, wird der Dateiname der Verknpfungsdatei (.lnk)
; zurckgegeben. Wird das Flag nicht angegeben, so wird der Name der Zieldatei
; der Verknpfung zurckgegeben.
; ########################################
Procedure.s SaveFileDialog(Title$, DefaultFile$, Pattern$, DefaultPattern, ParentWindowID = #Null, DefaultExtension$ = "", Flags = 0)
  
  Protected *FilterBuf, *c.Character, *FileBuf, OFN.OPENFILENAME, Result$

  ; Speicher fr Filter
  *FilterBuf = AllocateMemory((Len(Pattern$)+2)*SizeOf(Character)) ; Mit Len(Pattern$) + 2 wird sichergestellt, dass lpstrFilter mit zwei Null-Zeichen terminiert wird.
  If Not *FilterBuf
    Goto SaveFileDialog_Error
  EndIf  
  PokeS(*FilterBuf, Pattern$)
  *c = *FilterBuf
  While *c\c
    If *c\c = '|'
      *c\c = 0
    EndIf
    *c + SizeOf(Character)
  Wend

  ; Speicher fr Dateiname
  *FileBuf = AllocateMemory(#SaveFileDialog_FileBufferSize*SizeOf(Character))
  If Not *FileBuf
    Goto SaveFileDialog_Error
  EndIf
  PokeS(*FileBuf, DefaultFile$, #SaveFileDialog_FileBufferSize-1)
  
  ; Struktur initialisieren
  With OFN
    \lStructSize = SizeOf(OPENFILENAME)
    \hwndOwner = ParentWindowID
    \lpstrFilter = *FilterBuf
    \lpstrCustomFilter = #Null
    \nFilterIndex = DefaultPattern + 1
    \lpstrFile = *FileBuf
    \nMaxFile = #SaveFileDialog_FileBufferSize
    \lpstrFileTitle = #Null
    \lpstrInitialDir = #Null
    If Flags & #SaveFileDialog_DefaultTitle
      \lpstrTitle = #Null
    Else
      \lpstrTitle = @Title$
    EndIf
    If Not Flags & #SaveFileDialog_AddToRecent
      \Flags | #OFN_DONTADDTORECENT
    EndIf
    If Flags & #SaveFileDialog_OverwriteWarning
      \Flags | #OFN_OVERWRITEPROMPT
    EndIf
    If Flags & #SaveFileDialog_NoDereferenceLnk
      \Flags | #OFN_NODEREFERENCELINKS
    EndIf
    If DefaultExtension$ <> ""
      \lpstrDefExt = @DefaultExtension$
    Else
      \lpstrDefExt = #Null
    EndIf
  EndWith
  
  ; Dialog anzeigen
  If Not GetSaveFileName_(@OFN)
    Goto SaveFileDialog_Error
  EndIf
  
  ; Pattern ermitteln
  SaveFileDialog_SelectedPatternIndex = OFN\nFilterIndex - 1
  
  ; Dateiname ermitteln
  Result$ = PeekS(*FileBuf)

SaveFileDialog_Error:

  ; Speicher freigeben
  If *FilterBuf
    FreeMemory(*FilterBuf)
  EndIf
  If *FileBuf
    FreeMemory(*FileBuf)
  EndIf
  
  ; Dateiname zurckgeben
  ProcedureReturn Result$

EndProcedure



; ; Beispiele
; Debug OpenFileDialog("Pattern- und DefaultExtension", "C:\test.txt", "Text-Dateien|*.txt|HTML-Dateien|*.htm;*.html|Alle Dateien|*.*", 0, #Null, "xyz")
; Debug OpenFileDialogSelectedPatternIndex()
; Debug OpenFileDialog("#OpenFileDialog_DefaultTitle", "", "Alle Dateien|*.*", 0, #Null, "", #OpenFileDialog_DefaultTitle)
; Debug OpenFileDialog("#OpenFileDialog_AddToRecent", "", "Alle Dateien|*.*", 0, #Null, "", #OpenFileDialog_AddToRecent)
; Debug OpenFileDialog("#OpenFileDialog_ReadOnlyCheckBox|#OpenFileDialog_ReadOnlyChecked", "", "Alle Dateien|*.*", 0, #Null, "", #OpenFileDialog_ReadOnlyCheckBox|#OpenFileDialog_ReadOnlyChecked)
; Debug OpenFileDialogReadOnlyChecked()
; Debug OpenFileDialog("#OpenFileDialog_FileDoesntExistWarning", "", "Alle Dateien|*.*", 0, #Null, "", #OpenFileDialog_FileDoesntExistWarning)
; Debug OpenFileDialog("#OpenFileDialog_NoDereferenceLnk", "", "Alle Dateien|*.*", 0, #Null, "", #OpenFileDialog_NoDereferenceLnk)
; FileName$ = OpenFileDialog("#OpenFileDialog_MultiSelection", "", "Alle Dateien|*.*", 0, #Null, "", #OpenFileDialog_MultiSelection)
; While FileName$ <> ""
;   Debug FileName$
;   FileName$ = OpenFileDialogNextSelectedFileName()
; Wend
; ; Alternative Mglichkeit zur Ermittlung der ausgewhlten Dateinamen, bentigt keine temporre Stringvariable
; ForEach OpenFileDialog_FileList()
;   Debug OpenFileDialog_FileList()
; Next
; Debug SaveFileDialog("Pattern- und DefaultExtension", "C:\test.txt", "Text-Dateien|*.txt|HTML-Dateien|*.htm;*.html|Alle Dateien|*.*", 0, #Null, "xyz")
; Debug SaveFileDialogSelectedPatternIndex()
; Debug SaveFileDialog("#SaveFileDialog_DefaultTitle", "", "Alle Dateien|*.*", 0, #Null, "", #SaveFileDialog_DefaultTitle)
; Debug SaveFileDialog("#SaveFileDialog_AddToRecent", "", "Alle Dateien|*.*", 0, #Null, "", #SaveFileDialog_AddToRecent)
; Debug SaveFileDialog("#SaveFileDialog_OverwriteWarning", "", "Alle Dateien|*.*", 0, #Null, "", #SaveFileDialog_OverwriteWarning)
; Debug SaveFileDialog("#SaveFileDialog_NoDereferenceLnk", "", "Alle Dateien|*.*", 0, #Null, "", #SaveFileDialog_NoDereferenceLnk)



; IDE Options = PureBasic 5.10 (Windows - x86)
; Folding = --
; EnableXP