﻿; ########################################
; UTD2052CEL READER
; DSO-DATEIFUNKTIONEN
; (C) BY NILS HINTZE
; ########################################



; ########################################
;- Konstanten
; ########################################

; Triggermodus
Enumeration
  #DSO_TriggerMode_Auto
  #DSO_TriggerMode_Normal
  #DSO_TriggerMode_Single
EndEnumeration

; Triggerquelle
Enumeration
  #DSO_TriggerSource_Channel ; Kanal wird in TriggerSourceChannel angegeben
  #DSO_TriggerSource_EXT
  #DSO_TriggerSource_ACLine
  #DSO_TriggerSource_Alternate
EndEnumeration

; Triggerkopplung
Enumeration
  #DSO_TriggerCoupling_DC
  #DSO_TriggerCoupling_AC
  #DSO_TriggerCoupling_HFReject
  #DSO_TriggerCoupling_LFReject
EndEnumeration

; Triggertyp
Enumeration
  #DSO_TriggerType_Edge
  #DSO_TriggerType_Pulse
EndEnumeration

; Triggerflanke
Enumeration
  #DSO_TriggerSlope_Rise
  #DSO_TriggerSlope_Fall
  #DSO_TriggerSlope_RiseFall
EndEnumeration

; Triggerpolarität
Enumeration
  #DSO_TriggerPolarity_Positive
  #DSO_TriggerPolarity_Negative
EndEnumeration

; Trigger ><=
Enumeration
  #DSO_TriggerWhen_Greater
  #DSO_TriggerWhen_Lesser
  #DSO_TriggerWhen_Equal
EndEnumeration

; Kanäle
Enumeration
  #DSO_CH1
  #DSO_CH2
;   #DSO_CH3
;   #DSO_CH4
  #DSO_LastCHIndex = #PB_Compiler_EnumerationValue - 1
  #DSO_Math
  #DSO_LastChannelIndex = #PB_Compiler_EnumerationValue - 1
  #DSO_ChannelCount
EndEnumeration
; #DSO_LastCHIndex ist der Index des letzten "echten" Kanals in den Channel-Arrays.
; Um alle "echten" Kanäle durchzugehen, wird die folgende Schleife benutzt:
; For c = 0 to #DSO_LastCHIndex : Next
; #DSO_LastChannelIndex ist der Index des wirklich letzten Kanals in den Channel-Arrays.
; Dies ist zur Zeit der Math-Kanal.
; Um alle Kanäle inklusive des Math-Kanals durchzugehen, wird die folgende Schleife benutzt:
; For c = 0 to #DSO_LastChannelIndex : Next
; IsChannelCH() und IsChannelMath() können innerhalb der Schleife benutzt werden,
; um den Typ des aktuellen Kanals zu ermitteln.

; Kanal-Aktivierung
Enumeration
  #DSO_ChannelEnable_Off
  #DSO_ChannelEnable_On
EndEnumeration

; Kanal-Kopplung
Enumeration
  #DSO_ChannelCoupling_DC
  #DSO_ChannelCoupling_AC
  #DSO_ChannelCoupling_GND
EndEnumeration

; Kanal-Invertierung
Enumeration
  #DSO_ChannelInvert_Off
  #DSO_ChannelInvert_On ; Gibt an, dass die Werte in ChannelData bezogen auf ChannelOffset invertiert sind
EndEnumeration

; Kanal-Bandbreitenbegrenzung
Enumeration
  #DSO_ChannelBandwidthLimit_Off
  #DSO_ChannelBandwidthLimit_On
EndEnumeration

; Anzeigerelevanz-Flags
#DSO_Show_TimePerDiv        = $0001
#DSO_Show_TriggerMode       = $0002
#DSO_Show_TriggerSource     = $0004
#DSO_Show_TriggerCoupling   = $0008
#DSO_Show_TriggerLevel      = $0010
#DSO_Show_TriggerOffset     = $0020
#DSO_Show_TriggerType       = $0040
#DSO_Show_TriggerSlope      = $0080
#DSO_Show_TriggerPolarity   = $0100
#DSO_Show_TriggerWhen       = $0200
#DSO_Show_TriggerPulseWidth = $0400
#DSO_ChannelShow_VoltPerDiv       = $0001
#DSO_ChannelShow_Coupling         = $0002
#DSO_ChannelShow_ProbeAttenuation = $0004
#DSO_ChannelShow_Invert           = $0008
#DSO_ChannelShow_BandwidthLimit   = $0010



; ########################################
;- Strukturen
; ########################################

Structure ChannelData
  Value.d[0]
EndStructure

Structure DSOData
  FileName.s             ; Vollständiger Dateiname
  HorDivs.i              ; Horizontale Teilungen [Div]
  VertDivs.i             ; Vertikale Teilungen [Div]
  TimePerDiv.d           ; Zeitbasis [s/Div]
  TriggerMode.i          ; Triggermodus, eine #DSO_TriggerMode-Konstante
  TriggerSource.i        ; Triggerquelle, eine #DSO_TriggerSource-Konstante
  TriggerSourceChannel.i ; Triggerquellenkanal
  TriggerCoupling.i      ; Triggerkopplung, eine #DSO_TriggerCoupling-Konstante
  TriggerLevel.d         ; Triggerlevel [V]
  TriggerOffset.d        ; Triggeroffset [s]
  TriggerType.i          ; Triggertyp, eine #DSO_TriggerType-Konstante
  TriggerSlope.i         ; Triggerflanke, eine #DSO_TriggerSlope-Konstante
  TriggerPolarity.i      ; Triggerpolarität, eine #DSO_TriggerPolarity-Konstante
  TriggerWhen.i          ; Trigger ><=, eine #DSO_TriggerWhen-Konstante
  TriggerPulseWidth.d    ; Triggerpulsbreite [s]
  TriggerPoint.i         ; Triggerpunkt gemeinsam für alle aktiven Kanäle
  Show.i                 ; Anzeigerelevanz von Einstellungen
  PointsPerChannel.i     ; Datenpunktanzahl gemeinsam für alle aktiven Kanäle
  ChannelEnable.i[#DSO_ChannelCount]           ; Kanal-Aktivierung, eine #DSO_ChannelEnable-Konstante
  ChannelVoltPerDiv.d[#DSO_ChannelCount]       ; Kanal-Ablenkfaktor [V/Div], Kanal-Tastkopfteilung bereits enthalten
  ChannelCoupling.i[#DSO_ChannelCount]         ; Kanal-Kopplung, eine #DSO_ChannelCoupling-Konstante
  ChannelProbeAttenuation.d[#DSO_ChannelCount] ; Kanal-Tastkopfteilung [1]
  ChannelInvert.i[#DSO_ChannelCount]           ; Kanal-Invertierung, eine #DSO_ChannelInvert-Konstante
  ChannelBandwidthLimit.i[#DSO_ChannelCount]   ; Kanal-Bandbreitenbegrenzung, eine #DSO_ChannelBandwidthLimit-Konstante
  *ChannelDataMemory                           ; Kanal-Daten-Speicher
  *ChannelData.ChannelData[#DSO_ChannelCount]  ; Kanal-Daten [Div], Kanal-Offset bereits enthalten
  ChannelOffset.d[#DSO_ChannelCount]           ; Kanal-Offset [Div]
  ChannelShow.i[#DSO_ChannelCount]             ; Anzeigerelevanz von Kanal-Einstellungen
EndStructure



; ########################################
;- Prozeduren
; ########################################

; Prüfen, ob Datei von einem kompatiblen DSO stammt
Procedure IsUTD2052CELFile(File)
  Result = #False
  *Buffer = AllocateMemory(10)
  If *Buffer
    FileSeek(File, $000A)
    ReadData(File, *Buffer, 10)
    If CompareMemory(*Buffer, ?UTD2052CELIdentifier, 10) Or CompareMemory(*Buffer, ?UTD2102CELIdentifier, 10) Or CompareMemory(*Buffer, ?UTD2025CLIdentifier, 10)
      Result = #True
    EndIf
    FreeMemory(*Buffer)
  EndIf
  ProcedureReturn Result
  DataSection
    UTD2052CELIdentifier:
    Data.a $55, $54, $44, $32, $30, $35, $32, $43, $45, $4C
    UTD2102CELIdentifier:
    Data.a $55, $54, $44, $32, $31, $30, $32, $43, $45, $4C
    UTD2025CLIdentifier:
    Data.a $55, $54, $44, $32, $30, $32, $35, $43, $4C, $00
  EndDataSection
EndProcedure

; Kanal-Daten-Speicher reservieren
Procedure AllocateDSOChannelDataMemory(*DSOData.DSOData)
  If *DSOData\ChannelDataMemory ; Speicher freigeben, falls bereits reserviert
    FreeMemory(*DSOData\ChannelDataMemory)
  EndIf
  *DSOData\ChannelDataMemory = AllocateMemory(#DSO_ChannelCount**DSOData\PointsPerChannel*SizeOf(Double))
  If *DSOData\ChannelDataMemory
    For c = 0 To #DSO_LastChannelIndex
      *DSOData\ChannelData[c] = *DSOData\ChannelDataMemory + (c * *DSOData\PointsPerChannel * SizeOf(Double))
    Next
    ProcedureReturn #True
  EndIf
  ProcedureReturn #False
EndProcedure

; Alle Ressourcen freigeben
Procedure FreeDSOData(*DSOData.DSOData)
  If *DSOData\ChannelDataMemory
    FreeMemory(*DSOData\ChannelDataMemory)
    *DSOData\ChannelDataMemory = 0
  EndIf
EndProcedure

; Prüfen, ob der Kanal existiert
Macro IsChannel(Channel)
  ((Channel) >= 0 And (Channel) <= #DSO_LastChannelIndex)
EndMacro

; Prüfen, ob der Kanal ein "echter" Kanal ist
Macro IsChannelCH(Channel)
  ((Channel) >= 0 And (Channel) <= #DSO_LastCHIndex)
EndMacro

; Prüfen, ob der Kanal der Math-Kanal ist
Macro IsChannelMath(Channel)
  ((Channel) = #DSO_Math)
EndMacro

; Zeit eines Datenpunkts ermitteln.
Procedure.d DataPointTime(*DSOData.DSOData, DataPoint)
  ProcedureReturn *DSOData\HorDivs * *DSOData\TimePerDiv * DataPoint / *DSOData\PointsPerChannel
EndProcedure

; Gemessene Spannung an einem Datenpunkt ermitteln.
Procedure.d DataPointVoltage(*DSOData.DSOData, Channel, DataPoint)
  Voltage.d = (*DSOData\ChannelData[Channel]\Value[DataPoint] - *DSOData\ChannelOffset[Channel]) * *DSOData\ChannelVoltPerDiv[Channel]
  If *DSOData\ChannelInvert[Channel] = #DSO_ChannelInvert_On
    ProcedureReturn -Voltage
  EndIf
  ProcedureReturn Voltage
EndProcedure

; Mathematische Operation durchführen
#DSOMath_Value = -1
Enumeration
  #DSOMath_Disable ; Math-Kanal deaktivieren
  #DSOMath_Add ; Addition
  #DSOMath_Subtract ; Subtraktion
  #DSOMath_Multiply ; Multiplikation
  #DSOMath_Divide ; Division
  #DSOMath_Reciprocal ; Kehrwert
  #DSOMath_Square ; Quadrat
  #DSOMath_Sqrt ; Quadratwurzel
  #DSOMath_Abs ; Absolutwert
  #DSOMath_Invert ; Invertierung
EndEnumeration
Procedure DSOMath(*DSOData.DSOData, Operation, xOperand = #DSOMath_Value, yOperand = #DSOMath_Value, VoltPerDiv.d = 0, Offset.d = 0, x.d = 0, y.d = 0)
  
  ; Math-Kanal deaktivieren
  If Operation = #DSOMath_Disable
    *DSOData\ChannelEnable[#DSO_Math] = #DSO_ChannelEnable_Off
    ProcedureReturn
  EndIf
  
  ; Berechnung
  For n = 0 To *DSOData\PointsPerChannel - 1
    If xOperand <> #DSOMath_Value
      x = DataPointVoltage(*DSOData, xOperand, n)
    EndIf
    If yOperand <> #DSOMath_Value
      y = DataPointVoltage(*DSOData, yOperand, n)
    EndIf
    Select Operation
      Case #DSOMath_Add
        *DSOData\ChannelData[#DSO_Math]\Value[n] = x + y
      Case #DSOMath_Subtract
        *DSOData\ChannelData[#DSO_Math]\Value[n] = x - y
      Case #DSOMath_Multiply
        *DSOData\ChannelData[#DSO_Math]\Value[n] = x * y
      Case #DSOMath_Divide
        *DSOData\ChannelData[#DSO_Math]\Value[n] = x / y
      Case #DSOMath_Reciprocal
        *DSOData\ChannelData[#DSO_Math]\Value[n] = 1 / x
      Case #DSOMath_Square
        *DSOData\ChannelData[#DSO_Math]\Value[n] = Pow(x, 2)
      Case #DSOMath_Sqrt
        *DSOData\ChannelData[#DSO_Math]\Value[n] = Sqr(x)
      Case #DSOMath_Abs
        *DSOData\ChannelData[#DSO_Math]\Value[n] = Abs(x)
      Case #DSOMath_Invert
        *DSOData\ChannelData[#DSO_Math]\Value[n] = -x
    EndSelect
  Next
  
  ; Autorange
  If VoltPerDiv = 0
    Max.d = 0
    For n = 0 To *DSOData\PointsPerChannel - 1
      Value.d = Abs(*DSOData\ChannelData[#DSO_Math]\Value[n])
      If Value > Max
        Max = Value
      EndIf
    Next
    If Max = 0
      VoltPerDiv = 1
    Else
      VoltPerDiv = 2 * Max / (*DSOData\VertDivs * 3 / 4)
    EndIf
    Offset = 0
  EndIf
  
  ; Ablenkfaktor und Offset
  For n = 0 To *DSOData\PointsPerChannel - 1
    *DSOData\ChannelData[#DSO_Math]\Value[n] / VoltPerDiv
    *DSOData\ChannelData[#DSO_Math]\Value[n] + Offset
  Next
  
  ; Math-Kanal konfigurieren
  *DSOData\ChannelEnable[#DSO_Math] = #DSO_ChannelEnable_On
  *DSOData\ChannelVoltPerDiv[#DSO_Math] = VoltPerDiv
  *DSOData\ChannelOffset[#DSO_Math] = Offset

EndProcedure

; Datei lesen
Enumeration
  #ReadDSODataFile_Success
  #ReadDSODataFile_NoFileError
  #ReadDSODataFile_FormatError
  #ReadDSODataFile_MemoryError
  #ReadDSODataFile_IncorrectValueError
EndEnumeration
Procedure ReadDSODataFile(*DSOData.DSOData, FileName$)
  
  ; Datei öffnen
  File = ReadFile(#PB_Any, FileName$)
  
  If File

    If IsUTD2052CELFile(File)
    
      ; Rückgabewert, zunächst von Erfolg ausgehen
      Result = #ReadDSODataFile_Success
      
      ; Dateiname
      *DSOData\FileName = FileName$

      ; Unterteilung der Anzeige in Div
      *DSOData\HorDivs = 12
      *DSOData\VertDivs = 8
     
      ; Aktive Kanäle ermitteln, es kann nur CH1 allein oder CH1 und CH2 zusammen aktiv sein!
      FileSeek(File, $0061)
      Select ReadAsciiCharacter(File)
        Case $01
          *DSOData\ChannelEnable[#DSO_CH1] = #DSO_ChannelEnable_On
        Case $02
          *DSOData\ChannelEnable[#DSO_CH1] = #DSO_ChannelEnable_On
          *DSOData\ChannelEnable[#DSO_CH2] = #DSO_ChannelEnable_On
        Default
          Result = #ReadDSODataFile_IncorrectValueError
      EndSelect
      If Result <> #ReadDSODataFile_Success
        Goto ReadDSODataFile_Error
      EndIf
      
      ; CH1
      If *DSOData\ChannelEnable[#DSO_CH1] = #DSO_ChannelEnable_On
      
        ; Kopplung
        FileSeek(File, $007D)
        Select ReadAsciiCharacter(File)
          Case $00
            *DSOData\ChannelCoupling[#DSO_CH1] = #DSO_ChannelCoupling_DC
          Case $01
            *DSOData\ChannelCoupling[#DSO_CH1] = #DSO_ChannelCoupling_AC
          Case $02
            *DSOData\ChannelCoupling[#DSO_CH1] = #DSO_ChannelCoupling_GND
        EndSelect
        
        ; Bandbreitenbegrenzung
        FileSeek(File, $007E)
        Select ReadAsciiCharacter(File)
          Case $00
            *DSOData\ChannelBandwidthLimit[#DSO_CH1] = #DSO_ChannelBandwidthLimit_Off
          Case $01
            *DSOData\ChannelBandwidthLimit[#DSO_CH1] = #DSO_ChannelBandwidthLimit_On
        EndSelect
        
        ; Tastkopfteilung
        FileSeek(File, $0087)
        Select ReadAsciiCharacter(File)
          Case $00
            *DSOData\ChannelProbeAttenuation[#DSO_CH1] = 1
          Case $01
            *DSOData\ChannelProbeAttenuation[#DSO_CH1] = 10
          Case $02
            *DSOData\ChannelProbeAttenuation[#DSO_CH1] = 100
          Case $03
            *DSOData\ChannelProbeAttenuation[#DSO_CH1] = 1000
        EndSelect
        
        ; Invertierung
        FileSeek(File, $0088)
        Select ReadAsciiCharacter(File)
          Case $00
            *DSOData\ChannelInvert[#DSO_CH1] = #DSO_ChannelInvert_Off
          Case $01
            *DSOData\ChannelInvert[#DSO_CH1] = #DSO_ChannelInvert_On
        EndSelect
        
        ; Offset
        FileSeek(File, $0089)
        *DSOData\ChannelOffset[#DSO_CH1] = (ReadAsciiCharacter(File) - 128) / 25
        
        ; Ablenkfaktor
        FileSeek(File, $008B)
        *DSOData\ChannelVoltPerDiv[#DSO_CH1] = ReadQuad(File) * *DSOData\ChannelProbeAttenuation[#DSO_CH1] / 1e6
        
        ; Zeit/Datenpunkt
        FileSeek(File, $00A3)
        CH1TimePerPoint.q = ReadQuad(File)
        
        ; Datenbereichslänge
        FileSeek(File, $00AB)
        CH1PointCount = ReadUnicodeCharacter(File)
        
        ; Triggerpunkt
        FileSeek(File, $00AF)
        CH1TriggerPoint = ReadUnicodeCharacter(File)
        
        ; Festlegen, welche Kanal-Einstellungen relevant für die Anzeige sind
        *DSOData\ChannelShow[#DSO_CH1] = #DSO_ChannelShow_Coupling | #DSO_ChannelShow_BandwidthLimit | #DSO_ChannelShow_ProbeAttenuation | #DSO_ChannelShow_Invert | #DSO_ChannelShow_VoltPerDiv
      
      EndIf
           
      ; CH2
      If *DSOData\ChannelEnable[#DSO_CH2] = #DSO_ChannelEnable_On
        
        ; CH2 Offset ermitteln
        CH2Offset = $00BA + (CH1PointCount * 2)
      
        ; Kopplung
        FileSeek(File, CH2Offset+$0001)
        Select ReadAsciiCharacter(File)
          Case $00
            *DSOData\ChannelCoupling[#DSO_CH2] = #DSO_ChannelCoupling_DC
          Case $01
            *DSOData\ChannelCoupling[#DSO_CH2] = #DSO_ChannelCoupling_AC
          Case $02
            *DSOData\ChannelCoupling[#DSO_CH2] = #DSO_ChannelCoupling_GND
        EndSelect
        
        ; Bandbreitenbegrenzung
        FileSeek(File, CH2Offset+$0002)
        Select ReadAsciiCharacter(File)
          Case $00
            *DSOData\ChannelBandwidthLimit[#DSO_CH2] = #DSO_ChannelBandwidthLimit_Off
          Case $01
            *DSOData\ChannelBandwidthLimit[#DSO_CH2] = #DSO_ChannelBandwidthLimit_On
        EndSelect
        
        ; Tastkopfteilung
        FileSeek(File, CH2Offset+$000B)
        Select ReadAsciiCharacter(File)
          Case $00
            *DSOData\ChannelProbeAttenuation[#DSO_CH2] = 1
          Case $01
            *DSOData\ChannelProbeAttenuation[#DSO_CH2] = 10
          Case $02
            *DSOData\ChannelProbeAttenuation[#DSO_CH2] = 100
          Case $03
            *DSOData\ChannelProbeAttenuation[#DSO_CH2] = 1000
        EndSelect
        
        ; Invertierung
        FileSeek(File, CH2Offset+$000C)
        Select ReadAsciiCharacter(File)
          Case $00
            *DSOData\ChannelInvert[#DSO_CH2] = #DSO_ChannelInvert_Off
          Case $01
            *DSOData\ChannelInvert[#DSO_CH2] = #DSO_ChannelInvert_On
        EndSelect
        
        ; Offset
        FileSeek(File, CH2Offset+$000D)
        *DSOData\ChannelOffset[#DSO_CH2] = (ReadAsciiCharacter(File) - 128) / 25
        
        ; Ablenkfaktor
        FileSeek(File, CH2Offset+$000F)
        *DSOData\ChannelVoltPerDiv[#DSO_CH2] = ReadQuad(File) * *DSOData\ChannelProbeAttenuation[#DSO_CH2] / 1e6
        
        ; Zeit/Datenpunkt
        FileSeek(File, CH2Offset+$0027)
        CH2TimePerPoint.q = ReadQuad(File)
        
        ; Datenbereichslänge
        FileSeek(File, CH2Offset+$002F)
        CH2PointCount = ReadUnicodeCharacter(File)
        
        ; Triggerpunkt
        FileSeek(File, CH2Offset+$0033)
        CH2TriggerPoint = ReadUnicodeCharacter(File)
        
        ; Festlegen, welche Kanal-Einstellungen relevant für die Anzeige sind
        *DSOData\ChannelShow[#DSO_CH2] = #DSO_ChannelShow_Coupling | #DSO_ChannelShow_BandwidthLimit | #DSO_ChannelShow_ProbeAttenuation | #DSO_ChannelShow_Invert | #DSO_ChannelShow_VoltPerDiv

      EndIf
      
      ; Werte von CH1 und CH2 auf Gleichheit prüfen
      If *DSOData\ChannelEnable[#DSO_CH1] = #DSO_ChannelEnable_On And *DSOData\ChannelEnable[#DSO_CH2] = #DSO_ChannelEnable_On ; CH1 und CH2 sind aktiv
        If CH2PointCount <> CH1PointCount Or CH2TimePerPoint <> CH1TimePerPoint Or CH2TriggerPoint <> CH1TriggerPoint ; Datenpunktanzahl, Zeit/Datenpunkt, Triggerpunkte ungleich
          Result = #ReadDSODataFile_IncorrectValueError
          Goto ReadDSODataFile_Error
        EndIf
      EndIf

      ; Datenpunktanzahl
      *DSOData\PointsPerChannel = CH1PointCount
      
      ; Speicher allokieren
      If Not AllocateDSOChannelDataMemory(*DSOData)
        Result = #ReadDSODataFile_MemoryError
        Goto ReadDSODataFile_Error
      EndIf
      
      ; CH1 Daten lesen
      If *DSOData\ChannelEnable[#DSO_CH1] = #DSO_ChannelEnable_On
        FileSeek(File, $00BA)
        For n = 0 To *DSOData\PointsPerChannel - 1
          *DSOData\ChannelData[#DSO_CH1]\Value[n] = (ReadUnicodeCharacter(File) - 128) / 25
        Next
      EndIf
    
      ; CH2 Daten lesen
      If *DSOData\ChannelEnable[#DSO_CH2] = #DSO_ChannelEnable_On
        FileSeek(File, CH2Offset+$003E)
        For n = 0 To *DSOData\PointsPerChannel - 1
          *DSOData\ChannelData[#DSO_CH2]\Value[n] = (ReadUnicodeCharacter(File) - 128) / 25
        Next
      EndIf
      
      ; Zeitbasis
      *DSOData\TimePerDiv = (CH1TimePerPoint * *DSOData\PointsPerChannel / *DSOData\HorDivs) / 1e12
      *DSOData\Show = #DSO_Show_TimePerDiv

      ; Triggermodus
      FileSeek(File, $003E)
      Select ReadAsciiCharacter(File)
        Case $00
          *DSOData\TriggerMode = #DSO_TriggerMode_Auto
        Case $01
          *DSOData\TriggerMode = #DSO_TriggerMode_Normal
        Case $02
          *DSOData\TriggerMode = #DSO_TriggerMode_Single
      EndSelect
      
      ; Triggerquelle
      FileSeek(File, $003F)
      Select ReadAsciiCharacter(File)
        Case $00
          *DSOData\TriggerSource = #DSO_TriggerSource_Channel
          *DSOData\TriggerSourceChannel = #DSO_CH1
        Case $01
          *DSOData\TriggerSource = #DSO_TriggerSource_Channel
          *DSOData\TriggerSourceChannel = #DSO_CH2
        Case $04
          *DSOData\TriggerSource = #DSO_TriggerSource_EXT
        Case $06
          *DSOData\TriggerSource = #DSO_TriggerSource_ACLine
        Case $07
          *DSOData\TriggerSource = #DSO_TriggerSource_Alternate
      EndSelect
      
      ; Triggertyp
      FileSeek(File, $0048)
      Select ReadAsciiCharacter(File)
        Case $00
          *DSOData\TriggerType = #DSO_TriggerType_Edge
        Case $02
          *DSOData\TriggerType = #DSO_TriggerType_Pulse
      EndSelect
      
      ; Triggerkopplung
      FileSeek(File, $0049)
      Select ReadAsciiCharacter(File)
        Case $00
          *DSOData\TriggerCoupling = #DSO_TriggerCoupling_DC
        Case $01
          *DSOData\TriggerCoupling = #DSO_TriggerCoupling_AC
        Case $02
          *DSOData\TriggerCoupling = #DSO_TriggerCoupling_HFReject
        Case $03
          *DSOData\TriggerCoupling = #DSO_TriggerCoupling_LFReject
      EndSelect
      
      ; Triggerflanke
      FileSeek(File, $004A)
      Select ReadAsciiCharacter(File)
        Case $00
          *DSOData\TriggerSlope = #DSO_TriggerSlope_Rise
        Case $01
          *DSOData\TriggerSlope = #DSO_TriggerSlope_Fall
        Case $02
          *DSOData\TriggerSlope = #DSO_TriggerSlope_RiseFall
      EndSelect
      
      ; Triggerpulsbreite
      FileSeek(File, $004B)
      *DSOData\TriggerPulseWidth = ReadQuad(File) / 1e9
      
      ; Triggerpolarität
      FileSeek(File, $0053)
      Select ReadAsciiCharacter(File)
        Case $00
          *DSOData\TriggerPolarity = #DSO_TriggerPolarity_Positive
        Case $01
          *DSOData\TriggerPolarity = #DSO_TriggerPolarity_Negative
      EndSelect
      
      ; Trigger ><=
      FileSeek(File, $0054)
      Select ReadAsciiCharacter(File)
        Case $00
          *DSOData\TriggerWhen = #DSO_TriggerWhen_Greater
        Case $01
          *DSOData\TriggerWhen = #DSO_TriggerWhen_Lesser
        Case $02
          *DSOData\TriggerWhen = #DSO_TriggerWhen_Equal
      EndSelect
      
      ; Triggerlevel
      FileSeek(File, $0059)
      *DSOData\TriggerLevel = ReadQuad(File) / 1e6
      If *DSOData\TriggerSource = #DSO_TriggerSource_Channel
        *DSOData\TriggerLevel * *DSOData\ChannelProbeAttenuation[*DSOData\TriggerSourceChannel]
      EndIf
      
      ; Triggeroffset
      FileSeek(File, $0093)
      *DSOData\TriggerOffset = ReadQuad(File) / 1e12
      
      ; Triggerpunkt
      *DSOData\TriggerPoint = CH1TriggerPoint
      
      ; Festlegen, welche Trigger-Einstellungen relevant für die Anzeige sind
      *DSOData\Show | #DSO_Show_TriggerMode | #DSO_Show_TriggerSource | #DSO_Show_TriggerCoupling | #DSO_Show_TriggerLevel | #DSO_Show_TriggerOffset | #DSO_Show_TriggerType
      Select *DSOData\TriggerType
        Case #DSO_TriggerType_Edge
          *DSOData\Show | #DSO_Show_TriggerSlope
        Case #DSO_TriggerType_Pulse
          *DSOData\Show | #DSO_Show_TriggerPolarity | #DSO_Show_TriggerWhen | #DSO_Show_TriggerPulseWidth
      EndSelect
      
ReadDSODataFile_Error:
      
      ; Wenn Fehler aufgetreten, alle Ressourcen freigeben
      If Result <> #ReadDSODataFile_Success
        FreeDSOData(*DSOData)
      EndIf
      
    Else
      Result = #ReadDSODataFile_FormatError
    EndIf

    CloseFile(File)
    
  Else
    Result = #ReadDSODataFile_NoFileError
  EndIf
  
  ProcedureReturn Result
  
EndProcedure



; IDE Options = PureBasic 5.31 (Windows - x86)
; Folding = Aw
; EnableXP
; UseMainFile = utd2052celreader.pb